#define DIRTREE_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>
#include <Xm/Container.h>
#include <Xm/IconG.h>
#include <Xm/ScrolledW.h>

#include <pwd.h>
#include <sys/types.h>
#include <stdlib.h>

#include <unistd.h>

#include "fof.h"
#include "fileview.h"
#include "filelist.h"
#include "dirlist.h"
#include "interface.h"
#include "config.h"
#include "sort.h"
#include "pixmap.h"
#include "error_handler.h"
#include "windows.h"

extern void SelectionCB(Widget, XtPointer, XtPointer);
extern void GetLargestCellSize(TViewNode *filelist, int *width, int *height);

typedef struct {
  WidgetList w_icon;
  int nb_icons;
  int nb_icons_managed;
} TDirIconList;

struct DirIconNode {
  TDirNode *dir;
  int icon_index;

  struct DirIconNode *parent;
  
  int nb_children;
  struct DirIconNode **child_icon;
};

TDirIconList DirIconList;

typedef struct DirIconNode TDirIconNode;

TDirNode *RootDir;
TDirIconNode *RootIcon;

TDirNode *ActualDir;
TDirIconNode *ActualIcon;

TFileNode *ActualFileNodeList;
TViewNode *ActualFileViewList;

extern uid_t UserID;
extern gid_t GroupID;

int LastLargestCellWidth, LastLargestCellHeight;


XmStringTable GetDirAccess(TDirNode *dir)
{
  char *access;
  XmStringTable strings;

  strings = (XmStringTable) XtMalloc(sizeof(XmString));
  if(strings == NULL) {
    NonFatalError("dirtree.c","GetAccessString","Unable to Create details StringTable (XtMalloc failed)");
    return(NULL);
  }

  /* Get Access String */
  
  access = (char *) XtMalloc(sizeof(char)*7);
  if(access == NULL) {
    NonFatalError("dirtree.c","GetAccessString","Unable to Create Access String (XtMalloc failed)");
    XtFree((char*)strings);
    return(NULL);
  }

  if( ((dir->dir.UID==UserID) && (dir->dir.user & READ)) ||
      ((dir->dir.GID==GroupID) && (dir->dir.group & READ)) ||
      (dir->dir.others & READ)) access[0]='r'; else access[0]='-';

  access[1]='-';

  if( ((dir->dir.UID==UserID) && (dir->dir.user & WRITE)) ||
      ((dir->dir.GID==GroupID) && (dir->dir.group & WRITE)) ||
      (dir->dir.others & WRITE)) access[2]='w'; else access[2]='-';

  access[3]='-';

  if( ((dir->dir.UID==UserID) && (dir->dir.user & EXECUTE)) ||
      ((dir->dir.GID==GroupID) && (dir->dir.group & EXECUTE)) ||
      (dir->dir.others & EXECUTE)) access[4]='x'; else access[4]='-';

  access[5]=' ';
  access[6]=(char) 0;

  strings[0]= XmStringCreateSimple(access);
  if(strings[0]==NULL) {
    NonFatalError("dirtree.c","GetAccessString","Unable to Create Access String (XmStringCreateSimple failed)");
    XtFree(access);
    XtFree((char*)strings);
    return(NULL);
  }
  
  XtFree(access);
  return(strings);
}

int CreateDirIcon(Arg args[], int nbargs)
{
  /* if there is no more pre-allocated buffer */
  if(DirIconList.nb_icons_managed==DirIconList.nb_icons) {
    DirIconList.w_icon= (WidgetList) XtRealloc((char *) DirIconList.w_icon, sizeof(Widget)*DirIconList.nb_icons+1);
    DirIconList.w_icon[DirIconList.nb_icons]= XmCreateIconGadget(W_TreeContainer, "diricon", args, nbargs);
    XtManageChild(DirIconList.w_icon[DirIconList.nb_icons]);
    DirIconList.nb_icons++;
    DirIconList.nb_icons_managed++;
    return(DirIconList.nb_icons-1);
  } else {
    XtSetValues(DirIconList.w_icon[DirIconList.nb_icons_managed], args, nbargs);
    XtManageChild(DirIconList.w_icon[DirIconList.nb_icons_managed]);
    DirIconList.nb_icons_managed++;
    return(DirIconList.nb_icons_managed-1);
  }
}


TDirIconNode *CreateRootIcon(TDirNode *dir)
{
  TDirIconNode *node;
  Arg args[10];
  int nbargs;
  XmString string;
  XmStringTable access;

  node= (TDirIconNode *) XtMalloc(sizeof(TDirIconNode));
  if(node==NULL) FatalError("dirtree.c","CreateRootIcon","Unable to Create New Icon (XtMalloc failed)");

  node->dir = dir;

  node->parent = NULL;

  node->nb_children=0;
  node->child_icon=NULL;

  nbargs=0;
  XtSetArg(args[nbargs], XmNentryParent, NULL);nbargs++;
  string=NULL;
  string = XmStringCreateSimple("/");
  if(string==NULL) NonFatalError("dirtree.c","CreateRootIcon","Cannot Allocate XmString (XmStringCreateSimple failed)");
  access = GetDirAccess(node->dir);
  XtSetArg(args[nbargs], XmNlabelString, string);nbargs++;
  XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_opened);nbargs++;
  XtSetArg(args[nbargs], XmNoutlineState, XmEXPANDED);nbargs++;
  if(access!=NULL) {
    XtSetArg(args[nbargs], XmNdetail, access);nbargs++;
    XtSetArg(args[nbargs], XmNdetailCount, 1);nbargs++;
  }

  node->icon_index = CreateDirIcon(args, nbargs);

  if(access!=NULL) {
    XmStringFree(access[0]);
    XtFree((char *) access);
  }
  XmStringFree(string);

  return(node);
}
  
void MakeDirIconOneLevel(TDirIconNode *actualnode)
{
  int i;
  Arg args[11];
  int nbargs;
  XmString string;
  XmStringTable access;

  nbargs=0;
  XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_opened);nbargs++;
  XtSetArg(args[nbargs], XmNoutlineState, XmEXPANDED),nbargs++;
  XtSetValues(DirIconList.w_icon[actualnode->icon_index], args, nbargs);

  actualnode->nb_children=actualnode->dir->nb_children;
  
  actualnode->child_icon=(TDirIconNode **) XtMalloc(sizeof(TDirIconNode *)*actualnode->nb_children);
  if(actualnode->child_icon==NULL) FatalError("dirtree.c","MakeDirIconOneLevel","Unable to Create New Icon Children (XtMalloc failed)");
  for(i=0;i<actualnode->nb_children;i++) {
    actualnode->child_icon[i]= (TDirIconNode *) XtMalloc(sizeof(TDirIconNode));
    if(actualnode->child_icon[i]==NULL)  FatalError("dirtree.c","MakeDirIconOneLevel","Unable to Create New Icon Child (XtMalloc failed)");

    actualnode->child_icon[i]->dir=actualnode->dir->child_list[i];
    actualnode->child_icon[i]->parent=actualnode;
    actualnode->child_icon[i]->nb_children=0;
    actualnode->child_icon[i]->child_icon=NULL;

    nbargs=0;
    string=NULL;
    string = XmStringCreateSimple(actualnode->dir->child_list[i]->dir.name);
    if(string==NULL)  NonFatalError("dirtree.c","MakeDirIconOneLevel","Cannot Create XmString (XmStringCreateSimple failed");
    access = GetDirAccess(actualnode->child_icon[i]->dir);
    /* icon label */
    XtSetArg(args[nbargs], XmNlabelString, string);nbargs++;
    /* icon parent */
    XtSetArg(args[nbargs], XmNentryParent, DirIconList.w_icon[actualnode->icon_index]);nbargs++;
    XtSetArg(args[nbargs], XmNpositionIndex, i);nbargs++;
    /* icon pixmap */
    if(!actualnode->child_icon[i]->dir->dir.readable) {
      XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_unreadable);nbargs++;
    }
    else {
      XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_closed);nbargs++;
    }
    /* details */
    if(access!=NULL) {
      XtSetArg(args[nbargs], XmNdetail, access);nbargs++;
      XtSetArg(args[nbargs], XmNdetailCount, 1);nbargs++;
    }

    actualnode->child_icon[i]->icon_index = CreateDirIcon(args, nbargs);

  if(access!=NULL) {
    XmStringFree(access[0]);
    XtFree((char *) access);
  }

    XmStringFree(string);

  }
}

void DeleteSubdirIconNode(TDirIconNode *parent)
{
  int nbargs;
  Arg args[2];
  int i;

  for(i=0;i<parent->nb_children;i++) {
    if(parent->child_icon[i]!=NULL) DeleteSubdirIconNode(parent->child_icon[i]);
    if(parent!=RootIcon) {
      nbargs=0;
      XtSetArg(args[nbargs], XmNentryParent, NULL);nbargs++;
      XtSetValues(DirIconList.w_icon[parent->child_icon[i]->icon_index], args, nbargs);
    }
    XtUnmanageChild(DirIconList.w_icon[parent->child_icon[i]->icon_index]);
    DirIconList.nb_icons_managed--;
    
    XtFree((char *) parent->child_icon[i]);
    parent->child_icon[i]=NULL;
  }
  XtFree((char *) parent->child_icon);
  parent->child_icon=NULL;
  parent->nb_children=0;

  nbargs=0;
  XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_closed);nbargs++;
  XtSetValues(DirIconList.w_icon[parent->icon_index], args, nbargs);
}
    
void DeleteRootDirIcon(TDirIconNode *node)
{
  DeleteSubdirIconNode(node);
  XtUnmanageChild(DirIconList.w_icon[node->icon_index]);
  DirIconList.nb_icons_managed--;
  XtFree((char *)node);
}

TDirIconNode *SearchIconNode(Widget w_icon, TDirIconNode *rootnode)
{
  int i;
  TDirIconNode *result;

  if(DirIconList.w_icon[rootnode->icon_index]==w_icon) return(rootnode);
  if(rootnode->nb_children==0) return(NULL);

  result=NULL;
  i=0;
  do {
    result= SearchIconNode(w_icon, rootnode->child_icon[i]);
    i++;
  } while(i<rootnode->nb_children && result==NULL);
  if(result==NULL) return(NULL);

  return(result);
}

char **SaveSelectViewList(int *nbdirs)
{
  char **result;
  TViewNode *actual;
  int nbselected;
  int i;
  int sizepath;
  char *path;

  *nbdirs=0;

  /* count the selected files */

  nbselected=0;
  actual=SelectFileViewList;
  while(actual!=NULL) {
    nbselected++;
    actual=actual->next;
  }

  if(nbselected==0) return NULL;

  /* get the full pathname of the selected files */

  result= (char **) XtMalloc(sizeof(char *)*nbselected);

  path = ActualDir->dir.pathname;
  sizepath= strlen(path)+1;

  actual=SelectFileViewList;
  for(i=0;i<nbselected;i++) {
    result[i]= (char *) XtMalloc(sizeof(char)*(sizepath+strlen(actual->filenode->file.name)+1));
    strcpy(result[i], path);
    strcat(result[i],"/");
    strcat(result[i],actual->filenode->file.name); 
    actual=actual->next;
  }

  *nbdirs=nbselected;
  
  return result;
}

void TryToRestoreSelectViewList(char **names, int count) 
{
  TViewNode *actual;
  char *pathname;
  char *name;
  int i;
  WidgetList selection;
  int nbselection;
  XmContainerSelectCallbackStruct * infos;

  pathname=ActualDir->dir.pathname;

  /* select the matching files */

  selection=(WidgetList) XtMalloc(sizeof(Widget)*count);

  nbselection=0;

  for(i=0;i<count;i++) {
    actual=ActualFileViewList;
    while(actual!=NULL) {
      name = XtMalloc(sizeof(char)*(strlen(pathname)+1+strlen(actual->filenode->file.name)+1));
      strcpy(name, pathname);
      strcat(name, "/");
      strcat(name, actual->filenode->file.name);

      if(strcmp(names[i], name)==0 && strcmp(name,"/..")!=0) {
	selection[nbselection]=FileIconList.w_icon[actual->icon_index];
	nbselection++;
      }

      XtFree(name);
      actual=actual->next;
    }
  }

  if(nbselection!=0) {
  
    XtVaSetValues(W_ListFile, XmNselectedObjectCount, nbselection, XmNselectedObjects, selection, NULL);
  
    infos = (XmContainerSelectCallbackStruct *) XtMalloc(sizeof(XmContainerSelectCallbackStruct));
    infos->selected_item_count = nbselection;
    infos->selected_items = selection;

    SelectionCB(NULL, NULL, infos);

    XmScrollVisible(W_ScrollList, FileIconList.w_icon[SelectFileViewList->icon_index], 10, 10);

  XtFree ((char *) infos);

  }

  XtFree((char *) selection);
  
}
       
void RefreshFileList(void)
{
  int width, height;
  Bool auto_spacing;
  int default_width;
  WidgetList wselected;
  TDirIconNode *founddir;
  char **saveselect;
  int nbfilesaved;
  int i;
  XmString xstring;
  Widget w_temp;

  /* first replace the page in top/left */
  if(ActualFileViewList!=NULL) 
    /* if the first entry has an icon */
    if(ActualFileViewList->icon_index!=-1)
      XmScrollVisible(W_ScrollList, FileIconList.w_icon[ActualFileViewList->icon_index], 10000, 10000);
    /* else use the second one */
    else XmScrollVisible(W_ScrollList, FileIconList.w_icon[ActualFileViewList->next->icon_index], 10000, 10000);

  saveselect=NULL;

  /* remove selection */
  if(SelectFileViewList!=NULL) {
    saveselect=SaveSelectViewList(&nbfilesaved);
    SelectFileViewList=DeleteSelectViewList(SelectFileViewList);
  }

  XtVaSetValues(W_ListFile, XmNselectedObjects, NULL, XmNselectedObjectCount, 0, NULL);

  if(ActualFileNodeList!=NULL) {
    DestroyFileList(ActualFileNodeList);
    ActualFileNodeList=NULL;
  }
  if(ActualFileViewList!=NULL) {
    DeleteFileViewList(ActualFileViewList);
    ActualFileViewList=NULL;
  }
  
  WriteMessageLabel("Reading directory - please wait");
  
  ActualFileNodeList= CreateFileList(ActualDir);
  ActualFileViewList = CreateFileViewList(ActualFileNodeList);

  GetLargestCellSize(ActualFileViewList, &width, &height);
  LastLargestCellWidth=width;
  LastLargestCellHeight=height;

  GetVarBool("FileIconAutoSpacing", &auto_spacing);
  if(auto_spacing) {
    if(ActualViewMode==LARGE)
      XtVaSetValues(W_ListFile, XmNlargeCellWidth, width, XmNsmallCellHeight, height, NULL);
    else 
      XtVaSetValues(W_ListFile, XmNsmallCellWidth, width, XmNsmallCellHeight, height, NULL      );
  }
  else if(GetVarInt("FileIconDefaultWidth", &default_width)) {
    if(ActualViewMode==LARGE)
      XtVaSetValues(W_ListFile, XmNlargeCellWidth, default_width, NULL);
    else 
      XtVaSetValues(W_ListFile, XmNsmallCellWidth, default_width, NULL);
  }

  FixFileViewSize(NULL, NULL, NULL);
  /*  XmContainerRelayout(W_ListFile);*/
  
  /*test */  SortRoutine();

  /* refresh the dir tree */

  founddir = SearchIconNode(DirIconList.w_icon[ActualIcon->icon_index], RootIcon);
  if(founddir!=NULL) {
    DeleteSubdirNode(founddir->dir);
    DeleteSubdirIconNode(founddir);
    wselected = (WidgetList) XtMalloc(sizeof(Widget));
    wselected[0]=DirIconList.w_icon[ActualIcon->icon_index];
    XtVaSetValues(W_TreeContainer, XmNselectedObjects, wselected, XmNselectedObjectCount, 1, NULL);
    if(founddir!=NULL && founddir->dir->dir.readable==1) {
      ActualIcon = founddir;
      ActualDir = ActualIcon->dir;    
    }
    XtFree((char *) wselected);

     if(/*founddir!=RootIcon && */founddir!=NULL && founddir->dir->dir.readable==1 && founddir->dir->dir.opened==0) {
      ActualIcon = founddir;
      ActualDir = ActualIcon->dir;
      
      MakeDirOneLevel(ActualDir);
      MakeDirIconOneLevel(ActualIcon);
     }
  }

  /* trying to reselect the files */

  if(saveselect!=NULL) {
    TryToRestoreSelectViewList(saveselect, nbfilesaved);
    for(i=0;i<nbfilesaved;i++)
      XtFree((char *) saveselect[i]);
    XtFree((char *) saveselect);
  }

  /* show the current path */

  w_temp = XtNameToWidget(W_MainWindow, "*CurrentDirLabel");
  if(strcmp(ActualDir->dir.pathname,"")==0) 
    xstring=XmStringCreateSimple("/");
  else xstring=XmStringCreateSimple(ActualDir->dir.pathname);
  XtVaSetValues(w_temp, XmNlabelString, xstring, NULL);
  XmStringFree(xstring);


}

void TreeDirCloseCB(Widget w_container, XtPointer app_data, XtPointer widget_data)
{
  XmContainerOutlineCallbackStruct *infos;
  TDirIconNode *founddir;
  WidgetList wselected;

  infos= (XmContainerOutlineCallbackStruct *) widget_data;
  if(infos->event!=NULL) {
    if(infos->reason==XmCR_COLLAPSED) {
      founddir = SearchIconNode(infos->item, RootIcon);
    
      if(founddir!=NULL) {
	DeleteSubdirNode(founddir->dir);
	DeleteSubdirIconNode(founddir);
	wselected = (WidgetList) XtMalloc(sizeof(Widget));
	wselected[0]=infos->item;
	XtVaSetValues(W_TreeContainer, XmNselectedObjects, wselected, XmNselectedObjectCount, 1, NULL);
	if(founddir!=NULL && founddir->dir->dir.readable==1) {
	  ActualIcon = founddir;
	  ActualDir = ActualIcon->dir;
	  
	  ChangeWindowProperty();

	  RefreshFileList();
	  InitRefreshProc();
	}
	
	XtFree((char *) wselected); 
      }
    } else if(infos->reason==XmCR_EXPANDED) {
      founddir = SearchIconNode(infos->item, RootIcon);
      
      if(founddir==RootIcon) {
	ActualIcon = founddir;
	ActualDir = ActualIcon->dir;

	ChangeWindowProperty();
	
	MakeDirOneLevel(ActualDir);
	MakeDirIconOneLevel(ActualIcon);
      }
    }
  }
}

void TreeDirOpenCB(Widget w_container, XtPointer app_data, XtPointer widget_data)
{
  XmContainerSelectCallbackStruct *infos;
  TDirIconNode *founddir;
  Widget w_select;

  infos = (XmContainerSelectCallbackStruct *) widget_data;
  if(infos->reason==XmCR_DEFAULT_ACTION && infos->selected_item_count==1) {
    w_select = infos->selected_items[0];
    founddir = SearchIconNode(w_select, RootIcon);

    if(/*founddir!=RootIcon && */founddir!=NULL && founddir->dir->dir.readable==1 && founddir->dir->dir.opened==0) {
      ActualIcon = founddir;
      ActualDir = ActualIcon->dir;
      
      MakeDirOneLevel(ActualDir);
      MakeDirIconOneLevel(ActualIcon);
    }
  }
} 

void TreeDirSelectCB(Widget w_container, XtPointer app_data, XtPointer widget_data)
{
  XmContainerSelectCallbackStruct *infos;
  TDirIconNode *founddir;
  Widget w_select;

  infos = (XmContainerSelectCallbackStruct *) widget_data;
  if(infos->selected_item_count==0) {
    /* nothing happens when no dir is selected */

    /*if(ActualFileNodeList!=NULL) {
       DestroyFileList(ActualFileNodeList);
       ActualFileNodeList=NULL;
     }
     if(ActualFileViewList!=NULL) {
       DeleteFileViewList(ActualFileViewList);
       ActualFileViewList=NULL;
    }*/
  }
  else if(infos->reason==XmCR_SINGLE_SELECT) {
    w_select = infos->selected_items[0];
    founddir = SearchIconNode(w_select, RootIcon);

    if(founddir==ActualIcon && ActualFileViewList!=NULL) founddir=NULL;
    if(founddir!=NULL && founddir->dir->dir.readable==1) {
      ActualIcon = founddir;
      ActualDir = ActualIcon->dir;

      ChangeWindowProperty();

      RefreshFileList();
      InitRefreshProc();

    }
  }
} 
  
void InstallDirTreeCB(void)
{
  XtAddCallback(W_TreeContainer, XmNoutlineChangedCallback, TreeDirCloseCB, NULL);
  XtAddCallback(W_TreeContainer, XmNdefaultActionCallback, TreeDirOpenCB, NULL);
  XtAddCallback(W_TreeContainer, XmNselectionCallback, TreeDirSelectCB, NULL);
}

void CreateDirTree(char *cwd_string)
{
  char *temp_string;
  char newdir[257];
  char listdir[257];
  int i;
  int stopflag;
  Arg args[2];
  int nbargs;
  WidgetList w_select;

  RootDir = CreateRootNode();
  RootIcon = CreateRootIcon(RootDir);

  ActualDir = RootDir;
  ActualIcon = RootIcon;

  if(strcmp(cwd_string, "/") ==0) return; /* Ok, root directory */
  while(strcmp(cwd_string,"")!=0) {
    i=1;
    while(cwd_string[i]!='/' && cwd_string[i]!=(char)0) {
      newdir[i-1]= cwd_string[i];
      i++;
    }
    newdir[i-1]=(char)0;
    
    temp_string= (char *) XtMalloc(sizeof(char)*(strlen(cwd_string)+1 - i));
    if(temp_string==NULL) FatalError("dirtree.c","CreatreDirTree","Cannot allocate temporary string (XtMalloc failed)");
    strcpy(temp_string, &(cwd_string[i]));
    XtFree(cwd_string);
    cwd_string = temp_string;

    MakeDirOneLevel(ActualDir);
    MakeDirIconOneLevel(ActualIcon);
    
    stopflag=0;
    i=0;
    do {
      strcpy(listdir,ActualDir->child_list[i]->dir.name);
      if(strcmp(newdir, listdir)==0) {
	ActualDir=ActualDir->child_list[i];
	ActualIcon=ActualIcon->child_icon[i];
	stopflag=1;
      } else i++;
      if(!stopflag && i>ActualDir->nb_children) FatalError("dirtree.c","CreatreDirTree","Unable to find directory");
    } while(!stopflag);
  }     

  w_select = NULL;
  w_select = (WidgetList) XtMalloc(sizeof(Widget));
  if(w_select==NULL) FatalError("dirtree.c","CreatreDirTree","Cannot allocate Selected Widget List (XtMalloc failed)");
  w_select[0]=DirIconList.w_icon[ActualIcon->icon_index];
  nbargs=0;
  XtSetArg(args[nbargs], XmNselectedObjects, w_select);nbargs++;
  XtSetArg(args[nbargs], XmNselectedObjectCount, 1);nbargs++;
  XtSetValues(W_TreeContainer, args, nbargs);
  
  WriteMessageLabel("Reading directory - please wait");

  XmScrollVisible(W_ScrollTree, w_select[0], 10, 10);
  
  XtFree((char *) w_select);

  ChangeWindowProperty();

}

void RefreshTreeRoutine(void)
{
  char *cwd_string;

  DeleteSubdirNode(RootDir);
  DeleteRootDir(RootDir);

  DeleteSubdirIconNode(RootIcon);
  DeleteRootDirIcon(RootIcon);

  cwd_string = getcwd(NULL, 0);
  if(cwd_string==NULL) FatalError("fof.c","TimeOutStartProc","Cannot get current work directory (getcwd failed)");
  CreateDirTree(cwd_string);

  RefreshFileList();
  InitRefreshProc();

  ChangeWindowProperty();

}

void HomeDirRoutine(void)
{
  struct passwd *pwd;

  pwd = getpwuid(UserID);

  if(pwd!=NULL) {
    DeleteSubdirNode(RootDir);
    DeleteRootDir(RootDir);
    
    DeleteSubdirIconNode(RootIcon);
    DeleteRootDirIcon(RootIcon);

    CreateDirTree(pwd->pw_dir);
    
    RefreshFileList();
    InitRefreshProc();

    ChangeWindowProperty();

  } else MessageDialog("Home Dir", "Cannot get home directory");
}
    
int GoToDirRefreshRoutine(char *destdir)
{
  int result;

  result = chdir(destdir);
  if(result!=0) {
    MessageDialog("Go To Dir", "Cannot change directory");
    return -1;
  }
  else {
    result=access(destdir,R_OK);
    if(result!=0) {
      MessageDialog("Go To Dir", "Cannot change directory");
      return -1;
    }
    result=access(destdir,X_OK);
    if(result!=0) {
      MessageDialog("Go To Dir", "Cannot change directory");
      return -1;
    }
  }
    
  RefreshTreeRoutine();
  return 0;
}

void GoToDirNewWindow(char *dir)
{
  char commandline[256];
  char * deflt;
  int result;

  GetVarString("FofDefaultCommandLine", &deflt);
  strcpy(commandline, deflt);
  strcat(commandline, " ");
  strcat(commandline, dir);
  strcat(commandline, " &");
  result=system(commandline);
  if(result!=0) MessageDialog("New fof","Cannot find fof executable in PATH");
}

void InitModuleDirTree(void)
{
  DirIconList.w_icon=NULL;
  DirIconList.nb_icons=0;
  DirIconList.nb_icons_managed=0;
}





