#define PIXMAP_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>
#include <X11/xpm.h>
#include "interface.h"
#include "config.h"

Pixmap PIX_DIR_opened, PIX_DIR_closed, PIX_DIR_unreadable, PIX_DIR_read_only;

Pixmap PIX_FILE_SMALL_exec, PIX_FILE_SMALL_regular, PIX_FILE_SMALL_sys, PIX_FILE_SMALL_linkdir, PIX_FILE_SMALL_linkfile, PIX_FILE_SMALL_unreadable;

Pixmap PIX_FILE_LARGE_exec, PIX_FILE_LARGE_regular, PIX_FILE_LARGE_sys, PIX_FILE_LARGE_linkdir, PIX_FILE_LARGE_linkfile, PIX_FILE_LARGE_unreadable, PIX_DIR_LARGE, PIX_DIR_LARGE_unreadable;

Pixmap LoadPixmap(Widget parent, char *name)
{
  Pixmap pix, mask;
  XpmAttributes attributes;
  int status;
  Display *dpy = XtDisplay(parent);
  XpmColorSymbol symbols[3];
  Pixel bg;
  char *try;
  int i;

  XtVaGetValues(parent, XmNdepth, &attributes.depth, XmNcolormap, &attributes.colormap, XmNbackground, &bg, NULL);

  symbols[0].name="background";
  symbols[0].value=NULL;
  symbols[0].pixel=bg;

  symbols[1].name="none";
  symbols[1].value=NULL;
  symbols[1].pixel=bg;

  symbols[2].name="None";
  symbols[2].value=NULL;
  symbols[2].pixel=bg;

  attributes.colorsymbols=symbols;
  attributes.numsymbols=3;

  attributes.visual=DefaultVisual(dpy, DefaultScreen(dpy));
  attributes.valuemask=XpmColorSymbols | XpmDepth | XpmColormap | XpmVisual;

  i=0;
  do {
    try=XtMalloc(strlen(G_PixmapPath[i])+strlen(name)+1);
    strcpy(try, G_PixmapPath[i]);
    strcat(try, name);
    status = XpmReadFileToPixmap(dpy, DefaultRootWindow(dpy), try, &pix, &mask, &attributes);
    if(mask) XFreePixmap(dpy, mask);
    XtFree(try);
    i++;
  } while(status!=XpmSuccess && i<G_NbPaths);
  if(status!=XpmSuccess) return(0);
  else return pix;
}

void CreateDefaultPixmaps(void)
{
  char *name;

  /* default pixmaps = 0 */
  PIX_DIR_opened=0;
  PIX_DIR_closed=0;
  PIX_DIR_unreadable=0;
  PIX_DIR_read_only=0;
  PIX_FILE_SMALL_exec=0;
  PIX_FILE_SMALL_regular=0;
  PIX_FILE_SMALL_sys=0;
  PIX_FILE_SMALL_linkdir=0;
  PIX_FILE_SMALL_linkfile=0;
  PIX_FILE_SMALL_unreadable=0;
  PIX_FILE_LARGE_exec=0;
  PIX_FILE_LARGE_regular=0;
  PIX_FILE_LARGE_sys=0;
  PIX_FILE_LARGE_linkdir=0;
  PIX_FILE_LARGE_linkfile=0;
  PIX_FILE_LARGE_unreadable=0;
  PIX_DIR_LARGE=0;
  PIX_DIR_LARGE_unreadable=0;

  /* Dir pixmaps */

  if(GetVarString("PixDirOpened",&name)) 
    PIX_DIR_opened=LoadPixmap(W_TreeContainer, name);
  if(GetVarString("PixDirClosed", &name))
    PIX_DIR_closed=LoadPixmap(W_TreeContainer, name);
  if(GetVarString("PixDirUnreadable", &name))
    PIX_DIR_unreadable=LoadPixmap(W_TreeContainer, name);
  if(GetVarString("PixDirReadOnly", &name))
    PIX_DIR_read_only=LoadPixmap(W_TreeContainer, name);
  
  /* small pixmaps */

  if(GetVarString("PixFileSmallExec",&name)) 
    PIX_FILE_SMALL_exec=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileSmallRegular",&name)) 
    PIX_FILE_SMALL_regular=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileSmallSys",&name)) 
    PIX_FILE_SMALL_sys=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileSmallLinkDir",&name)) 
    PIX_FILE_SMALL_linkdir=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileSmallLinkFile",&name)) 
    PIX_FILE_SMALL_linkfile=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileSmallUnreadable",&name)) 
    PIX_FILE_SMALL_unreadable=LoadPixmap(W_ListFile, name);

  /* large pixmaps */

  if(GetVarString("PixFileLargeExec",&name)) 
    PIX_FILE_LARGE_exec=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileLargeRegular",&name)) 
    PIX_FILE_LARGE_regular=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileLargeSys",&name)) 
    PIX_FILE_LARGE_sys=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileLargeLinkDir",&name)) 
    PIX_FILE_LARGE_linkdir=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileLargeLinkFile",&name)) 
    PIX_FILE_LARGE_linkfile=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixFileLargeUnreadable",&name)) 
    PIX_FILE_LARGE_unreadable=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixDirLarge",&name)) 
    PIX_DIR_LARGE=LoadPixmap(W_ListFile, name);
  if(GetVarString("PixDirLargeUnreadable",&name)) 
    PIX_DIR_LARGE_unreadable=LoadPixmap(W_ListFile, name);
}


